# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEAC3A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type AC3A model. The model represents the field-controlled alternator-rectifier excitation systems
      designated type AC3A. These excitation systems include an alternator main exciter with non-controlled
      rectifiers. The exciter employs self-excitation, and the voltage regulator power is derived from the exciter
      output voltage.  Therefore, this system has an additional nonlinearity, simulated by the use of a multiplier
      whose inputs are the voltage regulator command signal, Va, and the exciter output voltage, Efd, times KR.
      This model is applicable to excitation systems employing static voltage regulators. Reference: IEEE
      421.5-2005, 6.3.

    tb: Voltage regulator time constant (TB) (>= 0).  Typical value = 0.
    tc: Voltage regulator time constant (TC) (>= 0).  Typical value = 0.
    ka: Voltage regulator gain (KA) (> 0).  Typical value = 45,62.
    ta: Voltage regulator time constant (TA) (> 0).  Typical value = 0,013.
    vamax: Maximum voltage regulator output (VAMAX) (> 0).  Typical value = 1.
    vamin: Minimum voltage regulator output (VAMIN) (< 0).  Typical value = -0,95.
    te: Exciter time constant, integration rate associated with exciter control (TE) (> 0).  Typical value = 1,17.
    vemin: Minimum exciter voltage output (VEMIN) (<= 0).  Typical value = 0.
    kr: Constant associated with regulator and alternator field power supply (KR) (> 0).  Typical value = 3,77.
    kf: Excitation control system stabilizer gains (KF) (>= 0).  Typical value = 0,143.
    tf: Excitation control system stabilizer time constant (TF) (> 0).  Typical value = 1.
    kn: Excitation control system stabilizer gain (KN) (>= 0).  Typical value = 0,05.
    efdn: Value of Efd at which feedback gain changes (EFDN) (> 0).  Typical value = 2,36.
    kc: Rectifier loading factor proportional to commutating reactance (KC) (>= 0).  Typical value = 0,104.
    kd: Demagnetizing factor, a function of exciter alternator reactances (KD) (>= 0).  Typical value = 0,499.
    ke: Exciter constant related to self-excited field (KE).  Typical value = 1.
    vfemax: Exciter field current limit reference (VFEMAX) (>= 0).  Typical value = 16.
    ve1: Exciter alternator output voltages back of commutating reactance at which saturation is defined (VE1) (> 0).
      Typical value = 6,24.
    seve1: Exciter saturation function value at the corresponding exciter voltage, VE1, back of commutating reactance
      (SE[VE1]) (>= 0).  Typical value = 1,143.
    ve2: Exciter alternator output voltages back of commutating reactance at which saturation is defined (VE2) (> 0).
      Typical value = 4,68.
    seve2: Exciter saturation function value at the corresponding exciter voltage, VE2, back of commutating reactance
      (SE[VE2]) (>= 0).  Typical value = 0,1.
    """

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vemin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kn: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdn: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vfemax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .RegulatingCondEq import RegulatingCondEq


@dataclass(config=DataclassConfig)
class ExternalNetworkInjection(RegulatingCondEq):
    """
    This class represents the external network and it is used for IEC 60909 calculations.

    governorSCD: Power Frequency Bias. This is the change in power injection divided by the change in frequency and
      negated.  A positive value of the power frequency bias provides additional power injection upon a
      drop in frequency.
    maxP: Maximum active power of the injection.
    maxQ: Maximum reactive power limit. It is used for modelling of infeed for load flow exchange and not for short
      circuit modelling.
    minP: Minimum active power of the injection.
    minQ: Minimum reactive power limit. It is used for modelling of infeed for load flow exchange and not for short
      circuit modelling.
    ikSecond: Indicates whether initial symmetrical short-circuit current and power have been calculated according to
      IEC (Ik`).  Used only if short circuit calculations are done according to superposition method.
    maxInitialSymShCCurrent: Maximum initial symmetrical short-circuit currents (Ik` max) in A (Ik` = Sk`/(SQRT(3) Un)).
      Used for short circuit data exchange according to IEC 60909.
    maxR0ToX0Ratio: Maximum ratio of zero sequence resistance of Network Feeder to its zero sequence reactance
      (R(0)/X(0) max). Used for short circuit data exchange according to IEC 60909.
    maxR1ToX1Ratio: Maximum ratio of positive sequence resistance of Network Feeder to its positive sequence reactance
      (R(1)/X(1) max). Used for short circuit data exchange according to IEC 60909.
    maxZ0ToZ1Ratio: Maximum ratio of zero sequence impedance to its positive sequence impedance (Z(0)/Z(1) max). Used
      for short circuit data exchange according to IEC 60909.
    minInitialSymShCCurrent: Minimum initial symmetrical short-circuit currents (Ik` min) in A (Ik` = Sk`/(SQRT(3) Un)).
      Used for short circuit data exchange according to IEC 60909.
    minR0ToX0Ratio: Indicates whether initial symmetrical short-circuit current and power have been calculated according
      to IEC (Ik`). Used for short circuit data exchange according to IEC 6090.
    minR1ToX1Ratio: Minimum ratio of positive sequence resistance of Network Feeder to its positive sequence reactance
      (R(1)/X(1) min). Used for short circuit data exchange according to IEC 60909.
    minZ0ToZ1Ratio: Minimum ratio of zero sequence impedance to its positive sequence impedance (Z(0)/Z(1) min). Used
      for short circuit data exchange according to IEC 60909.
    voltageFactor: Voltage factor in pu, which was used to calculate short-circuit current Ik` and power Sk`.  Used only
      if short circuit calculations are done according to superposition method.
    referencePriority: Priority of unit for use as powerflow voltage phase angle reference bus selection. 0 = don t care
      (default) 1 = highest priority. 2 is less than 1 and so on.
    p: Active power injection. Load sign convention is used, i.e. positive sign means flow out from a node. Starting
      value for steady state solutions.
    q: Reactive power injection. Load sign convention is used, i.e. positive sign means flow out from a node. Starting
      value for steady state solutions.
    """

    governorSCD: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    maxP: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    maxQ: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    minP: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    minQ: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    ikSecond: bool = Field(
        default=False,
        in_profiles=[
            Profile.SC,
        ],
    )

    maxInitialSymShCCurrent: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    maxR0ToX0Ratio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    maxR1ToX1Ratio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    maxZ0ToZ1Ratio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    minInitialSymShCCurrent: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    minR0ToX0Ratio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    minR1ToX1Ratio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    minZ0ToZ1Ratio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    voltageFactor: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    referencePriority: int = Field(
        default=0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    p: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    q: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SC,
            Profile.SSH,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class Quality61850(Base):
    """
    Quality flags in this class are as defined in IEC 61850, except for estimatorReplaced, which has been included in
      this class for convenience.

    badReference: Measurement value may be incorrect due to a reference being out of calibration.
    estimatorReplaced: Value has been replaced by State Estimator. estimatorReplaced is not an IEC61850 quality bit but
      has been put in this class for convenience.
    failure: This identifier indicates that a supervision function has detected an internal or external failure, e.g.
      communication failure.
    oldData: Measurement value is old and possibly invalid, as it has not been successfully updated during a specified
      time interval.
    operatorBlocked: Measurement value is blocked and hence unavailable for transmission.
    oscillatory: To prevent some overload of the communication it is sensible to detect and suppress oscillating (fast
      changing) binary inputs. If a signal changes in a defined time twice in the same direction (from
      0 to 1 or from 1 to 0) then oscillation is detected and the detail quality identifier
      `oscillatory` is set. If it is detected a configured numbers of transient changes could be passed
      by. In this time the validity status `questionable` is set. If after this defined numbers of
      changes the signal is still in the oscillating state the value shall be set either to the
      opposite state of the previous stable value or to a defined default value. In this case the
      validity status `questionable` is reset and `invalid` is set as long as the signal is
      oscillating. If it is configured such that no transient changes should be passed by then the
      validity status `invalid` is set immediately in addition to the detail quality identifier
      `oscillatory` (used for status information only).
    outOfRange: Measurement value is beyond a predefined range of value.
    overFlow: Measurement value is beyond the capability of being  represented properly. For example, a counter value
      overflows from maximum count back to a value of zero.
    source: Source gives information related to the origin of a value. The value may be acquired from the process,
      defaulted or substituted.
    suspect: A correlation function has detected that the value is not consistent with other values. Typically set by a
      network State Estimator.
    test: Measurement value is transmitted for test purposes.
    validity: Validity of the measurement value.
    """

    badReference: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    estimatorReplaced: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    failure: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    oldData: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    operatorBlocked: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    oscillatory: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    outOfRange: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    overFlow: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    source: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.OP,
        ],
    )

    suspect: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    test: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    validity: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.OP,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class VisibilityLayer(IdentifiedObject):
    """
    Layers are typically used for grouping diagram objects according to themes and scales. Themes are used to display or
      hide certain information (e.g., lakes, borders), while scales are used for hiding or displaying information
      depending on the current zoom level (hide text when it is too small to be read, or when it exceeds the screen
      size). This is also called de-cluttering. CIM based graphics exchange supports an m:n relationship between
      diagram objects and layers. The importing system shall convert an m:n case into an appropriate 1:n
      representation if the importing system does not support m:n.

    VisibleObjects: A visibility layer can contain one or more diagram objects.
    drawingOrder: The drawing order for this layer.  The higher the number, the later the layer and the objects within
      it are rendered.
    """

    VisibleObjects: list = Field(
        default_factory=list,
        in_profiles=[
            Profile.DL,
        ],
    )

    drawingOrder: int = Field(
        default=0,
        in_profiles=[
            Profile.DL,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssPTIST3(PowerSystemStabilizerDynamics):
    """
    PTI microprocessor-based stabilizer type 3.

    m: (M).  M = 2 x H.  Typical value = 5.
    tf: Time constant (Tf) (>= 0).  Typical value = 0,2.
    tp: Time constant (Tp) (>= 0).  Typical value = 0,2.
    t1: Time constant (T1) (>= 0).  Typical value = 0,3.
    t2: Time constant (T2) (>= 0).  Typical value = 1.
    t3: Time constant (T3) (>= 0).  Typical value = 0,2.
    t4: Time constant (T4) (>= 0).  Typical value = 0,05.
    k: Gain (K).  Typical value = 9.
    dtf: Time step frequency calculation (deltatf) (>= 0).  Typical value = 0,025 (0,03 for 50 Hz).
    dtc: Time step related to activation of controls (deltatc) (>= 0).  Typical value = 0,025 (0,03 for 50 Hz).
    dtp: Time step active power calculation (deltatp) (>= 0).  Typical value = 0,0125  (0,015 for 50 Hz).
    t5: Time constant (T5) (>= 0).
    t6: Time constant (T6) (>= 0).
    a0: Filter coefficient (A0).
    a1: Limiter (Al).
    a2: Filter coefficient (A2).
    b0: Filter coefficient (B0).
    b1: Filter coefficient (B1).
    b2: Filter coefficient (B2).
    a3: Filter coefficient (A3).
    a4: Filter coefficient (A4).
    a5: Filter coefficient (A5).
    b3: Filter coefficient (B3).
    b4: Filter coefficient (B4).
    b5: Filter coefficient (B5).
    athres: Threshold value above which output averaging will be bypassed (Athres).  Typical value = 0,005.
    dl: Limiter (Dl).
    al: Limiter (Al).
    lthres: Threshold value (Lthres).
    pmin: (Pmin).
    isw: Digital/analogue output switch (Isw). true = produce analogue output false = convert to digital output, using
      tap selection table.
    nav: Number of control outputs to average (NAV) (1 <=  NAV <= 16).  Typical value = 4.
    ncl: Number of counts at limit to active limit function (NCL) (> 0).
    ncr: Number of counts until reset after limit function is triggered (NCR).
    """

    m: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dtf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dtc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dtp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    athres: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    al: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    lthres: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    isw: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    nav: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ncl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ncr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .LoadDynamics import LoadDynamics


@dataclass(config=DataclassConfig)
class LoadComposite(LoadDynamics):
    """
    Combined static load and induction motor load effects. The dynamics of the motor are simplified by linearizing the
      induction machine equations.

    epvs: Active load-voltage dependence index (static) (Epvs).  Typical value = 0,7.
    epfs: Active load-frequency dependence index (static) (Epfs).  Typical value = 1,5.
    eqvs: Reactive load-voltage dependence index (static) (Eqvs).  Typical value = 2.
    eqfs: Reactive load-frequency dependence index (static) (Eqfs).  Typical value = 0.
    epvd: Active load-voltage dependence index (dynamic) (Epvd).  Typical value = 0,7.
    epfd: Active load-frequency dependence index (dynamic) (Epfd).  Typical value = 1,5.
    eqvd: Reactive load-voltage dependence index (dynamic) (Eqvd).  Typical value = 2.
    eqfd: Reactive load-frequency dependence index (dynamic) (Eqfd).  Typical value = 0.
    lfac: Loading factor (Lfac). The ratio of initial P to motor MVA base.  Typical value = 0,8.
    h: Inertia constant (H) (>= 0).  Typical value = 2,5.
    pfrac: Fraction of constant-power load to be represented by this motor model (PFRAC) (>= 0,0 and <= 1,0).  Typical
      value = 0,5.
    """

    epvs: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    epfs: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eqvs: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eqfs: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    epvd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    epfd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eqvd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eqfd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    lfac: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    h: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pfrac: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TransformerEnd import TransformerEnd


@dataclass(config=DataclassConfig)
class PowerTransformerEnd(TransformerEnd):
    """
    A PowerTransformerEnd is associated with each Terminal of a PowerTransformer. The impedance values r, r0, x, and x0
      of a PowerTransformerEnd represents a star equivalent as follows. 1) for a two Terminal PowerTransformer the
      high voltage (TransformerEnd.endNumber=1) PowerTransformerEnd has non zero values on r, r0, x, and x0 while
      the low voltage (TransformerEnd.endNumber=2) PowerTransformerEnd has zero values for r, r0, x, and x0.
      Parameters are always provided, even if the PowerTransformerEnds have the same rated voltage.  In this case,
      the parameters are provided at the PowerTransformerEnd which has TransformerEnd.endNumber equal to 1. 2) for a
      three Terminal PowerTransformer the three PowerTransformerEnds represent a star equivalent with each leg in
      the star represented by r, r0, x, and x0 values. 3) For a three Terminal transformer each PowerTransformerEnd
      shall have g, g0, b and b0 values corresponding to the no load losses distributed on the three
      PowerTransformerEnds. The total no load loss shunt impedances may also be placed at one of the
      PowerTransformerEnds, preferably the end numbered 1, having the shunt values on end 1.  This is the preferred
      way. 4) for a PowerTransformer with more than three Terminals the PowerTransformerEnd impedance values cannot
      be used. Instead use the TransformerMeshImpedance or split the transformer into multiple PowerTransformers.
      Each PowerTransformerEnd must be contained by a PowerTransformer. Because a PowerTransformerEnd (or any other
      object) can not be contained by more than one parent, a PowerTransformerEnd can not have an association to an
      EquipmentContainer (Substation, VoltageLevel, etc).

    PowerTransformer: The power transformer of this power transformer end.
    b: Magnetizing branch susceptance (B mag).  The value can be positive or negative.
    connectionKind: Kind of connection.
    ratedS: Normal apparent power rating. The attribute shall be a positive value. For a two-winding transformer the
      values for the high and low voltage sides shall be identical.
    g: Magnetizing branch conductance.
    ratedU: Rated voltage: phase-phase for three-phase windings, and either phase-phase or phase-neutral for single-
      phase windings. A high voltage side, as given by TransformerEnd.endNumber, shall have a ratedU that is
      greater than or equal to ratedU for the lower voltage sides. The attribute shall be a positive value.
    r: Resistance (star-model) of the transformer end. The attribute shall be equal to or greater than zero for non-
      equivalent transformers.
    x: Positive sequence series reactance (star-model) of the transformer end.
    b0: Zero sequence magnetizing branch susceptance.
    phaseAngleClock: Terminal voltage phase angle displacement where 360 degrees are represented with clock hours. The
      valid values are 0 to 11. For example, for the secondary side end of a transformer with
      vector group code of `Dyn11`, specify the connection kind as wye with neutral and specify the
      phase angle of the clock as 11.  The clock value of the transformer end number specified as
      1, is assumed to be zero.  Note the transformer end number is not assumed to be the same as
      the terminal sequence number.
    g0: Zero sequence magnetizing branch conductance (star-model).
    r0: Zero sequence series resistance (star-model) of the transformer end.
    x0: Zero sequence series reactance of the transformer end.
    """

    PowerTransformer: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    b: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    connectionKind: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    ratedS: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    g: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    ratedU: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    r: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    x: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    b0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    phaseAngleClock: int = Field(
        default=0,
        in_profiles=[
            Profile.SC,
        ],
    )

    g0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    r0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    x0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SC,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindAeroTwoDimIEC(IdentifiedObject):
    """
    Two-dimensional aerodynamic model.   Reference: IEC 61400-27-1:2015, 5.6.1.3.

    dpomega: Partial derivative of aerodynamic power with respect to changes in WTR speed (dpomega). It is a type-
      dependent parameter.
    dptheta: Partial derivative of aerodynamic power with respect to changes in pitch angle (dptheta). It is a type-
      dependent parameter.
    dpv1: Partial derivative (dpv1). It is a type-dependent parameter.
    omegazero: Rotor speed if the wind turbine is not derated (omega0). It is a type-dependent parameter.
    pavail: Available aerodynamic power (pavail). It is a case-dependent parameter.
    thetazero: Pitch angle if the wind turbine is not derated (theta0). It is a case-dependent parameter.
    thetav2: Blade angle at twice rated wind speed (thetav2). It is a type-dependent parameter.
    WindTurbineType3IEC: Wind turbine type 3 model with which this wind aerodynamic model is associated.
    """

    dpomega: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dptheta: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dpv1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    omegazero: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pavail: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetazero: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetav2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .Curve import Curve


@dataclass(config=DataclassConfig)
class VsCapabilityCurve(Curve):
    """
    The P-Q capability curve for a voltage source converter, with P on X-axis and Qmin and Qmax on Y1-axis and Y2-axis.

    VsConverterDCSides: All converters with this capability curve.
    """

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # VsConverterDCSides : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .UnderexcitationLimiterDynamics import UnderexcitationLimiterDynamics


@dataclass(config=DataclassConfig)
class UnderexcLimIEEE2(UnderexcitationLimiterDynamics):
    """
    Type UEL2 underexcitation limiter which has either a straight-line or multi-segment characteristic when plotted in
      terms of machine reactive power output vs. real power output. Reference: IEEE UEL2 421.5-2005, 10.2  (limit
      characteristic lookup table shown in Figure 10.4 (p 32)).

    tuv: Voltage filter time constant (TUV) (>= 0).  Typical value = 5.
    tup: Real power filter time constant (TUP) (>= 0).  Typical value = 5.
    tuq: Reactive power filter time constant (TUQ) (>= 0).  Typical value = 0.
    kui: UEL integral gain (KUI).  Typical value = 0,5.
    kul: UEL proportional gain (KUL).  Typical value = 0,8.
    vuimax: UEL integrator output maximum limit (VUIMAX) (> UnderexcLimIEEE2.vuimin).  Typical value = 0,25.
    vuimin: UEL integrator output minimum limit (VUIMIN) (< UnderexcLimIEEE2.vuimax).  Typical value = 0.
    kuf: UEL excitation system stabilizer gain (KUF).  Typical value = 0.
    kfb: Gain associated with optional integrator feedback input signal to UEL (KFB).  Typical value = 0.
    tul: Time constant associated with optional integrator feedback input signal to UEL (TUL) (>= 0).  Typical value =
      0.
    tu1: UEL lead time constant (TU1) (>= 0).  Typical value = 0.
    tu2: UEL lag time constant (TU2) (>= 0).  Typical value = 0.
    tu3: UEL lead time constant (TU3) (>= 0).  Typical value = 0.
    tu4: UEL lag time constant (TU4) (>= 0).  Typical value = 0.
    vulmax: UEL output maximum limit (VULMAX) (> UnderexcLimIEEE2.vulmin).  Typical value = 0,25.
    vulmin: UEL output minimum limit (VULMIN) (< UnderexcLimIEEE2.vulmax).  Typical value = 0.
    p0: Real power values for endpoints (P0).  Typical value = 0.
    q0: Reactive power values for endpoints (Q0).  Typical value = -0,31.
    p1: Real power values for endpoints (P1).  Typical value = 0,3.
    q1: Reactive power values for endpoints (Q1).  Typical value = -0,31.
    p2: Real power values for endpoints (P2).  Typical value = 0,6.
    q2: Reactive power values for endpoints (Q2).  Typical value = -0,28.
    p3: Real power values for endpoints (P3).  Typical value = 0,9.
    q3: Reactive power values for endpoints (Q3).  Typical value = -0,21.
    p4: Real power values for endpoints (P4).  Typical value = 1,02.
    q4: Reactive power values for endpoints (Q4).  Typical value = 0.
    p5: Real power values for endpoints (P5).
    q5: Reactive power values for endpoints (Q5).
    p6: Real power values for endpoints (P6).
    q6: Reactive power values for endpoints (Q6).
    p7: Real power values for endpoints (P7).
    q7: Reactive power values for endpoints (Q7).
    p8: Real power values for endpoints (P8).
    q8: Reactive power values for endpoints (Q8).
    p9: Real power values for endpoints (P9).
    q9: Reactive power values for endpoints (Q9).
    p10: Real power values for endpoints (P10).
    q10: Reactive power values for endpoints (Q10).
    k1: UEL terminal voltage exponent applied to real power input to UEL limit look-up table (k1).  Typical value = 2.
    k2: UEL terminal voltage exponent applied to reactive power output from UEL limit look-up table (k2).  Typical value
      = 2.
    """

    tuv: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tup: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tuq: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kui: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kul: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vuimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vuimin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kuf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kfb: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tul: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tu1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tu2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tu3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tu4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vulmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vulmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p7: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q7: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p8: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q8: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p9: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q9: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    p10: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    q10: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class FossilFuel(IdentifiedObject):
    """
    The fossil fuel consumed by the non-nuclear thermal generating unit.   For example, coal, oil, gas, etc.   These are
      the specific fuels that the generating unit can consume.

    fossilFuelType: The type of fossil fuel, such as coal, oil, or gas.
    ThermalGeneratingUnit: A thermal generating unit may have one or more fossil fuels.
    """

    fossilFuelType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    ThermalGeneratingUnit: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemResource import PowerSystemResource


@dataclass(config=DataclassConfig)
class Equipment(PowerSystemResource):
    """
    The parts of a power system that are physical devices, electronic or mechanical.

    EquipmentContainer: Container of this equipment.
    aggregate: The aggregate flag provides an alternative way of representing an aggregated (equivalent) element. It is
      applicable in cases when the dedicated classes for equivalent equipment do not have all of the
      attributes necessary to represent the required level of detail.  In case the flag is set to `true`
      the single instance of equipment represents multiple pieces of equipment that have been modelled
      together as an aggregate equivalent obtained by a network reduction procedure. Examples would be
      power transformers or synchronous machines operating in parallel modelled as a single aggregate
      power transformer or aggregate synchronous machine.   The attribute is not used for
      EquivalentBranch, EquivalentShunt and EquivalentInjection.
    normallyInService: Specifies the availability of the equipment under normal operating conditions. True means the
      equipment is available for topology processing, which determines if the equipment is
      energized or not. False means that the equipment is treated by network applications as if
      it is not in the model.
    OperationalLimitSet: The operational limit sets associated with this equipment.
    inService: Specifies the availability of the equipment. True means the equipment is available for topology
      processing, which determines if the equipment is energized or not. False means that the equipment
      is treated by network applications as if it is not in the model.
    """

    EquipmentContainer: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQBD,
            Profile.EQ,
        ],
    )

    aggregate: bool = Field(
        default=False,
        in_profiles=[
            Profile.EQ,
        ],
    )

    normallyInService: bool = Field(
        default=False,
        in_profiles=[
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # OperationalLimitSet : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    inService: bool = Field(
        default=False,
        in_profiles=[
            Profile.SSH,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssWECC(PowerSystemStabilizerDynamics):
    """
    Dual input power system stabilizer, based on IEEE type 2, with modified output limiter defined by WECC (Western
      Electricity Coordinating Council, USA).

    inputSignal1Type: Type of input signal #1 (rotorAngularFrequencyDeviation, busFrequencyDeviation,
      generatorElectricalPower, generatorAcceleratingPower, busVoltage, or busVoltageDerivative -
      shall be different than PssWECC.inputSignal2Type).  Typical value =
      rotorAngularFrequencyDeviation.
    inputSignal2Type: Type of input signal #2 (rotorAngularFrequencyDeviation, busFrequencyDeviation,
      generatorElectricalPower, generatorAcceleratingPower, busVoltage, busVoltageDerivative -
      shall be different than PssWECC.inputSignal1Type).  Typical value = busVoltageDerivative.
    k1: Input signal 1 gain (K1).  Typical value = 1,13.
    t1: Input signal 1 transducer time constant (T1) (>= 0).  Typical value = 0,037.
    k2: Input signal 2 gain (K2).  Typical value = 0,0.
    t2: Input signal 2 transducer time constant (T2) (>= 0).  Typical value = 0,0.
    t3: Stabilizer washout time constant (T3) (>= 0).  Typical value = 9,5.
    t4: Stabilizer washout time lag constant (T4) (>= 0).  Typical value = 9,5.
    t5: Lead time constant (T5) (>= 0).  Typical value = 1,7.
    t6: Lag time constant (T6) (>= 0).  Typical value = 1,5.
    t7: Lead time constant (T7) (>= 0).  Typical value = 1,7.
    t8: Lag time constant (T8) (>= 0).  Typical value = 1,5.
    t10: Lag time constant (T10) (>= 0).  Typical value = 0.
    t9: Lead time constant (T9) (>= 0).  Typical value = 0.
    vsmax: Maximum output signal (Vsmax) (> PssWECC.vsmin). Typical value = 0,05.
    vsmin: Minimum output signal (Vsmin) (< PssWECC.vsmax).  Typical value = -0,05.
    vcu: Maximum value for voltage compensator output (VCU). Typical value = 0.
    vcl: Minimum value for voltage compensator output (VCL). Typical value = 0.
    """

    inputSignal1Type: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    inputSignal2Type: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    k1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t10: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t9: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vcu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vcl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IOPoint import IOPoint


@dataclass(config=DataclassConfig)
class Control(IOPoint):
    """
    Control is used for supervisory/device control. It represents control outputs that are used to change the state in a
      process, e.g. close or open breaker, a set point value or a raise lower command.

    controlType: Specifies the type of Control. For example, this specifies if the Control represents BreakerOpen,
      BreakerClose, GeneratorVoltageSetPoint, GeneratorRaise, GeneratorLower, etc.
    operationInProgress: Indicates that a client is currently sending control commands that has not completed.
    timeStamp: The last time a control output was sent.
    unitMultiplier: The unit multiplier of the controlled quantity.
    unitSymbol: The unit of measure of the controlled quantity.
    PowerSystemResource: Regulating device governed by this control output.
    """

    controlType: str = Field(
        default="",
        in_profiles=[
            Profile.OP,
        ],
    )

    operationInProgress: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    timeStamp: str = Field(
        default="",
        in_profiles=[
            Profile.OP,
        ],
    )

    unitMultiplier: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.OP,
        ],
    )

    unitSymbol: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.OP,
        ],
    )

    PowerSystemResource: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.OP,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class UnitMultiplier(Base):
    """
    The unit multipliers defined for the CIM.  When applied to unit symbols, the unit symbol is treated as a derived
      unit. Regardless of the contents of the unit symbol text, the unit symbol shall be treated as if it were a
      single-character unit symbol. Unit symbols should not contain multipliers, and it should be left to the
      multiplier to define the multiple for an entire data type.   For example, if a unit symbol is "m2Pers" and the
      multiplier is "k", then the value is k(m**2/s), and the multiplier applies to the entire final value, not to
      any individual part of the value. This can be conceptualized by substituting a derived unit symbol for the
      unit type. If one imagines that the symbol "Þ" represents the derived unit "m2Pers", then applying the
      multiplier "k" can be conceptualized simply as "kÞ".  For example, the SI unit for mass is "kg" and not "g".
      If the unit symbol is defined as "kg", then the multiplier is applied to "kg" as a whole and does not replace
      the "k" in front of the "g". In this case, the multiplier of "m" would be used with the unit symbol of "kg" to
      represent one gram.  As a text string, this violates the instructions in IEC 80000-1. However, because the
      unit symbol in CIM is treated as a derived unit instead of as an SI unit, it makes more sense to conceptualize
      the "kg" as if it were replaced by one of the proposed replacements for the SI mass symbol. If one imagines
      that the "kg" were replaced by a symbol "Þ", then it is easier to conceptualize the multiplier "m" as creating
      the proper unit "mÞ", and not the forbidden unit "mkg".

    """

    # No attributes defined for this class.

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DL,
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssPTIST1(PowerSystemStabilizerDynamics):
    """
    PTI microprocessor-based stabilizer type 1.

    m: (M).  M = 2 x H.  Typical value = 5.
    tf: Time constant (Tf) (>= 0).  Typical value = 0,2.
    tp: Time constant (Tp) (>= 0).  Typical value = 0,2.
    t1: Time constant (T1) (>= 0).  Typical value = 0,3.
    t2: Time constant (T2) (>= 0).  Typical value = 1.
    t3: Time constant (T3) (>= 0).  Typical value = 0,2.
    t4: Time constant (T4) (>= 0).  Typical value = 0,05.
    k: Gain (K).  Typical value = 9.
    dtf: Time step frequency calculation (deltatf) (>= 0).  Typical value = 0,025.
    dtc: Time step related to activation of controls (deltatc) (>= 0).  Typical value = 0,025.
    dtp: Time step active power calculation (deltatp) (>= 0).  Typical value = 0,0125.
    """

    m: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dtf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dtc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dtp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .DynamicsFunctionBlock import DynamicsFunctionBlock


@dataclass(config=DataclassConfig)
class RotatingMachineDynamics(DynamicsFunctionBlock):
    """
    Abstract parent class for all synchronous and asynchronous machine standard models.

    damping: Damping torque coefficient (D) (>= 0).  A proportionality constant that, when multiplied by the angular
      velocity of the rotor poles with respect to the magnetic field (frequency), results in the damping
      torque.  This value is often zero when the sources of damping torques (generator damper windings,
      load damping effects, etc.) are modelled in detail.  Typical value = 0.
    inertia: Inertia constant of generator or motor and mechanical load (H) (> 0).  This is the specification for the
      stored energy in the rotating mass when operating at rated speed.  For a generator, this includes the
      generator plus all other elements (turbine, exciter) on the same shaft and has units of MW x s.  For
      a motor, it includes the motor plus its mechanical load. Conventional units are PU on the generator
      MVA base, usually expressed as MW x s / MVA or just s. This value is used in the accelerating power
      reference frame for operator training simulator solutions.  Typical value = 3.
    saturationFactor: Saturation factor at rated terminal voltage (S1) (>= 0).  Not used by simplified model.  Defined
      by defined by S(E1) in the SynchronousMachineSaturationParameters diagram.  Typical value =
      0,02.
    saturationFactor120: Saturation factor at 120% of rated terminal voltage (S12) (>=
      RotatingMachineDynamics.saturationFactor). Not used by the simplified model, defined by
      S(E2) in the SynchronousMachineSaturationParameters diagram.  Typical value = 0,12.
    statorLeakageReactance: Stator leakage reactance (Xl) (>= 0). Typical value = 0,15.
    statorResistance: Stator (armature) resistance (Rs) (>= 0). Typical value = 0,005.
    """

    damping: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    inertia: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    saturationFactor: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    saturationFactor120: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    statorLeakageReactance: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    statorResistance: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcST7B(ExcitationSystemDynamics):
    """
    Modified IEEE ST7B static excitation system without stator current limiter (SCL) and current compensator (DROOP)
      inputs.

    kh: High-value gate feedback gain (Kh) (>= 0).  Typical value = 1.
    kia: Voltage regulator integral gain (Kia) (>= 0).  Typical value = 1.
    kl: Low-value gate feedback gain (Kl) (>= 0).  Typical value = 1.
    kpa: Voltage regulator proportional gain (Kpa) (> 0).  Typical value = 40.
    oelin: OEL input selector (OELin). Typical value = noOELinput.
    tb: Regulator lag time constant (Tb) (>= 0).  Typical value = 1.
    tc: Regulator lead time constant (Tc) (>= 0).  Typical value = 1.
    tf: Excitation control system stabilizer time constant (Tf) (>= 0).  Typical value = 1.
    tg: Feedback time constant of inner loop field voltage regulator (Tg) (>= 0).  Typical value = 1.
    tia: Feedback time constant (Tia) (>= 0).  Typical value = 3.
    ts: Rectifier firing time constant (Ts) (>= 0).  Typical value = 0.
    uelin: UEL input selector (UELin). Typical value = noUELinput.
    vmax: Maximum voltage reference signal (Vmax) (> 0 and > ExcST7B.vmin)).  Typical value = 1,1.
    vmin: Minimum voltage reference signal (Vmin) (> 0 and < ExcST7B.vmax).  Typical value = 0,9.
    vrmax: Maximum voltage regulator output (Vrmax) (> 0).  Typical value = 5.
    vrmin: Minimum voltage regulator output (Vrmin) (< 0).  Typical value = -4,5.
    """

    kh: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kia: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpa: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    oelin: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tg: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tia: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uelin: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class RegularTimePoint(Base):
    """
    Time point for a schedule where the time between the consecutive points is constant.

    sequenceNumber: The position of the regular time point in the sequence. Note that time points don`t have to be
      sequential, i.e. time points may be omitted. The actual time for a RegularTimePoint is
      computed by multiplying the associated regular interval schedule`s time step with the regular
      time point sequence number and adding the associated schedules start time. To specify values
      for the start time, use sequence number 0.  The sequence number cannot be negative.
    value1: The first value at the time. The meaning of the value is defined by the derived type of the associated
      schedule.
    value2: The second value at the time. The meaning of the value is defined by the derived type of the associated
      schedule.
    IntervalSchedule: Regular interval schedule containing this time point.
    """

    sequenceNumber: int = Field(
        default=0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    value1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    value2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    IntervalSchedule: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovHydroR(TurbineGovernorDynamics):
    """
    Fourth order lead-lag governor and hydro turbine.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    pmax: Maximum gate opening, PU of MWbase (Pmax) (> GovHydroR.pmin).  Typical value = 1.
    pmin: Minimum gate opening, PU of MWbase (Pmin) (< GovHydroR.pmax).  Typical value = 0.
    r: Steady-state droop (R).  Typical value = 0,05.
    td: Input filter time constant (Td) (>= 0).  Typical value = 0,05.
    t1: Lead time constant 1 (T1) (>= 0).  Typical value = 1,5.
    t2: Lag time constant 1 (T2) (>= 0).  Typical value = 0,1.
    t3: Lead time constant 2 (T3) (>= 0).  Typical value = 1,5.
    t4: Lag time constant 2 (T4) (>= 0).  Typical value = 0,1.
    t5: Lead time constant 3 (T5) (>= 0).  Typical value = 0.
    t6: Lag time constant 3 (T6) (>= 0).  Typical value = 0,05.
    t7: Lead time constant 4 (T7) (>= 0).  Typical value = 0.
    t8: Lag time constant 4 (T8) (>= 0).  Typical value = 0,05.
    tp: Gate servo time constant (Tp) (>= 0).  Typical value = 0,05.
    velop: Maximum gate opening velocity (Velop).  Unit = PU / s.  Typical value = 0,2.
    velcl: Maximum gate closing velocity (Velcl).  Unit = PU / s.  Typical value = -0,2.
    ki: Integral gain (Ki).  Typical value = 0,5.
    kg: Gate servo gain (Kg).  Typical value = 2.
    gmax: Maximum governor output (Gmax) (> GovHydroR.gmin).  Typical value = 1,05.
    gmin: Minimum governor output (Gmin) (< GovHydroR.gmax).  Typical value = -0,05.
    tt: Power feedback time constant (Tt) (>= 0).  Typical value = 0.
    inputSignal: Input signal switch (Flag). true = Pe input is used false = feedback is received from CV. Flag is
      normally dependent on Tt.  If Tt is zero, Flag is set to false. If Tt is not zero, Flag is set to
      true.   Typical value = true.
    db1: Intentional dead-band width (db1).  Unit = Hz.  Typical value = 0.
    eps: Intentional db hysteresis (eps).  Unit = Hz.  Typical value = 0.
    db2: Unintentional dead-band (db2).  Unit = MW.  Typical value = 0.
    tw: Water inertia time constant (Tw) (> 0).  Typical value = 1.
    at: Turbine gain (At).  Typical value = 1,2.
    dturb: Turbine damping factor (Dturb).  Typical value = 0,2.
    qnl: No-load turbine flow at nominal head (Qnl).  Typical value = 0,08.
    h0: Turbine nominal head (H0).  Typical value = 1.
    gv1: Nonlinear gain point 1, PU gv (Gv1).  Typical value = 0.
    pgv1: Nonlinear gain point 1, PU power (Pgv1).  Typical value = 0.
    gv2: Nonlinear gain point 2, PU gv (Gv2).  Typical value = 0.
    pgv2: Nonlinear gain point 2, PU power (Pgv2).  Typical value = 0.
    gv3: Nonlinear gain point 3, PU gv (Gv3).  Typical value = 0.
    pgv3: Nonlinear gain point 3, PU power (Pgv3).  Typical value = 0.
    gv4: Nonlinear gain point 4, PU gv (Gv4).  Typical value = 0.
    pgv4: Nonlinear gain point 4, PU power (Pgv4).  Typical value = 0.
    gv5: Nonlinear gain point 5, PU gv (Gv5).  Typical value = 0.
    pgv5: Nonlinear gain point 5, PU power (Pgv5).  Typical value = 0.
    gv6: Nonlinear gain point 6, PU gv (Gv6).  Typical value = 0.
    pgv6: Nonlinear gain point 6, PU power (Pgv6).  Typical value = 0.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    r: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    td: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t7: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t8: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    velop: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    velcl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kg: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tt: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    inputSignal: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    db1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eps: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    db2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    at: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dturb: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    qnl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    h0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gv6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pgv6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PFVArControllerType1Dynamics import PFVArControllerType1Dynamics


@dataclass(config=DataclassConfig)
class PFVArType1IEEEVArController(PFVArControllerType1Dynamics):
    """
    IEEE VAR controller type 1 which operates by moving the voltage reference directly. Reference: IEEE 421.5-2005,
      11.3.

    tvarc: Var controller time delay (TVARC) (>= 0).  Typical value = 5.
    vvar: Synchronous machine power factor (VVAR).
    vvarcbw: Var controller deadband (VVARC_BW).  Typical value = 0,02.
    vvarref: Var controller reference (VVARREF).
    vvtmax: Maximum machine terminal voltage needed for pf/VAr controller to be enabled (VVTMAX) (>
      PVFArType1IEEEVArController.vvtmin).
    vvtmin: Minimum machine terminal voltage needed to enable pf/var controller (VVTMIN) (<
      PVFArType1IEEEVArController.vvtmax).
    """

    tvarc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvar: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvarcbw: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvarref: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvtmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvtmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEST7B(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type ST7B model. This model is representative of static potential-source excitation systems. In this
      system, the AVR consists of a PI voltage regulator. A phase lead-lag filter in series allows the introduction
      of a derivative function, typically used with brushless excitation systems. In that case, the regulator is of
      the PID type. In addition, the terminal voltage channel includes a phase lead-lag filter.  The AVR includes
      the appropriate inputs on its reference for overexcitation limiter (OEL1), underexcitation limiter (UEL),
      stator current limiter (SCL), and current compensator (DROOP). All these limitations, when they work at
      voltage reference level, keep the PSS (VS signal from PSS) in operation. However, the UEL limitation can also
      be transferred to the high value (HV) gate acting on the output signal. In addition, the output signal passes
      through a low value (LV) gate for a ceiling overexcitation limiter (OEL2). Reference: IEEE 421.5-2005, 7.7.

    kh: High-value gate feedback gain (KH) (>= 0).  Typical value = 1.
    kia: Voltage regulator integral gain (KIA) (>= 0).  Typical value = 1.
    kl: Low-value gate feedback gain (KL) (>= 0).  Typical value = 1.
    kpa: Voltage regulator proportional gain (KPA) (> 0).  Typical value = 40.
    oelin: OEL input selector (OELin).  Typical value = noOELinput.
    tb: Regulator lag time constant (TB) (>= 0).  Typical value = 1.
    tc: Regulator lead time constant (TC) (>= 0).  Typical value = 1.
    tf: Excitation control system stabilizer time constant (TF) (>= 0).  Typical value = 1.
    tg: Feedback time constant of inner loop field voltage regulator (TG) (>= 0). Typical value = 1.
    tia: Feedback time constant (TIA) (>= 0).  Typical value = 3.
    uelin: UEL input selector (UELin). Typical value = noUELinput.
    vmax: Maximum voltage reference signal (VMAX) (> 0 and > ExcIEEEST7B.vmin).  Typical value = 1,1.
    vmin: Minimum voltage reference signal (VMIN) (> 0 and < ExcIEEEST7B.vmax).  Typical value = 0,9.
    vrmax: Maximum voltage regulator output (VRMAX) (> 0).  Typical value = 5.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0).  Typical value = -4,5.
    """

    kh: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kia: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpa: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    oelin: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tg: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tia: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uelin: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TapChanger import TapChanger


@dataclass(config=DataclassConfig)
class PhaseTapChanger(TapChanger):
    """
    A transformer phase shifting tap model that controls the phase angle difference across the power transformer and
      potentially the active power flow through the power transformer.  This phase tap model may also impact the
      voltage magnitude.

    TransformerEnd: Transformer end to which this phase tap changer belongs.
    """

    TransformerEnd: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SSH,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class LoadStatic(IdentifiedObject):
    """
    General static load. This model represents the sensitivity of the real and reactive power consumed by the load to
      the amplitude and frequency of the bus voltage.

    LoadAggregate: Aggregate load to which this aggregate static load belongs.
    staticLoadModelType: Type of static load model.  Typical value = constantZ.
    kp1: First term voltage coefficient for active power (Kp1).  Not used when .staticLoadModelType = constantZ.
    kp2: Second term voltage coefficient for active power (Kp2).  Not used when .staticLoadModelType = constantZ.
    kp3: Third term voltage coefficient for active power (Kp3).  Not used when .staticLoadModelType = constantZ.
    kp4: Frequency coefficient for active power (Kp4)  (not = 0 if .staticLoadModelType = zIP2).  Used only when
      .staticLoadModelType = zIP2.
    ep1: First term voltage exponent for active power (Ep1).  Used only when .staticLoadModelType = exponential.
    ep2: Second term voltage exponent for active power (Ep2).  Used only when .staticLoadModelType = exponential.
    ep3: Third term voltage exponent for active power (Ep3).  Used only when .staticLoadModelType = exponential.
    kpf: Frequency deviation coefficient for active power (Kpf).  Not used when .staticLoadModelType = constantZ.
    kq1: First term voltage coefficient for reactive power (Kq1).  Not used when .staticLoadModelType = constantZ.
    kq2: Second term voltage coefficient for reactive power (Kq2).  Not used when .staticLoadModelType = constantZ.
    kq3: Third term voltage coefficient for reactive power (Kq3).  Not used when .staticLoadModelType = constantZ.
    kq4: Frequency coefficient for reactive power (Kq4)  (not = 0 when .staticLoadModelType = zIP2).  Used only when
      .staticLoadModelType - zIP2.
    eq1: First term voltage exponent for reactive power (Eq1).  Used only when .staticLoadModelType = exponential.
    eq2: Second term voltage exponent for reactive power (Eq2).  Used only when .staticLoadModelType = exponential.
    eq3: Third term voltage exponent for reactive power (Eq3).  Used only when .staticLoadModelType = exponential.
    kqf: Frequency deviation coefficient for reactive power (Kqf).  Not used when .staticLoadModelType = constantZ.
    """

    LoadAggregate: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    staticLoadModelType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ep1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ep2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ep3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kq1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kq2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kq3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kq4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eq1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eq2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    eq3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kqf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0


class DataclassConfig:  # pylint: disable=too-few-public-methods
    """
    Used to configure pydantic dataclasses.

    See doc at
    https://docs.pydantic.dev/latest/usage/model_config/#options
    """

    # By default, with pydantic extra arguments given to a dataclass are silently ignored.
    # This matches the default behaviour by failing noisily.
    extra = "forbid"



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .SynchronousMachineDynamics import SynchronousMachineDynamics


@dataclass(config=DataclassConfig)
class SynchronousMachineSimplified(SynchronousMachineDynamics):
    """
    The simplified model represents a synchronous generator as a constant internal voltage behind an impedance (Rs +
      jXp) as shown in the Simplified diagram. Since internal voltage is held constant, there is no Efd input and
      any excitation system model will be ignored.  There is also no Ifd output. This model should not be used for
      representing a real generator except, perhaps, small generators whose response is insignificant.   The
      parameters used for the simplified model include: - RotatingMachineDynamics.damping (D); -
      RotatingMachineDynamics.inertia (H); - RotatingMachineDynamics.statorLeakageReactance (used to exchange jXp
      for SynchronousMachineSimplified); - RotatingMachineDynamics.statorResistance (Rs).

    """

    # No attributes defined for this class.

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PFVArControllerType2Dynamics import PFVArControllerType2Dynamics


@dataclass(config=DataclassConfig)
class PFVArType2IEEEVArController(PFVArControllerType2Dynamics):
    """
    IEEE VAR controller type 2 which is a summing point type controller. It makes up the outside loop of a two-loop
      system. This controller is implemented as a slow PI type controller, and the voltage regulator forms the inner
      loop and is implemented as a fast controller. Reference: IEEE 421.5-2005, 11.5.

    qref: Reactive power reference (QREF).
    vref: Voltage regulator reference (VREF).
    vclmt: Maximum output of the pf controller (VCLMT).
    kp: Proportional gain of the pf controller (KP).
    ki: Integral gain of the pf controller (KI).
    vs: Generator sensing voltage (VS).
    exlon: Overexcitation or under excitation flag (EXLON) true = 1 (not in the overexcitation or underexcitation state,
      integral action is active) false = 0 (in the overexcitation or underexcitation state, so integral
      action is disabled to allow the limiter to play its role).
    """

    qref: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vref: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vclmt: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vs: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    exlon: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEAC5A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type AC5A model. The model represents a simplified model for brushless excitation systems. The
      regulator is supplied from a source, such as a permanent magnet generator, which is not affected by system
      disturbances.  Unlike other AC models, this model uses loaded rather than open circuit exciter saturation data
      in the same way as it is used for the DC models.  Because the model has been widely implemented by the
      industry, it is sometimes used to represent other types of systems when either detailed data for them are not
      available or simplified models are required. Reference: IEEE 421.5-2005, 6.5.

    ka: Voltage regulator gain (KA) (> 0).  Typical value = 400.
    ta: Voltage regulator time constant (TA) (> 0).  Typical value = 0,02.
    vrmax: Maximum voltage regulator output (VRMAX) (> 0).  Typical value = 7,3.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0).  Typical value = -7,3.
    ke: Exciter constant related to self-excited field (KE).  Typical value = 1.
    te: Exciter time constant, integration rate associated with exciter control (TE) (> 0).  Typical value = 0,8.
    kf: Excitation control system stabilizer gains (KF) (>= 0).  Typical value = 0,03.
    tf1: Excitation control system stabilizer time constant (TF1) (> 0).  Typical value = 1.
    tf2: Excitation control system stabilizer time constant (TF2) (>= 0).  Typical value = 1.
    tf3: Excitation control system stabilizer time constant (TF3) (>= 0).  Typical value = 1.
    efd1: Exciter voltage at which exciter saturation is defined (EFD1) (> 0).  Typical value = 5,6.
    seefd1: Exciter saturation function value at the corresponding exciter voltage, EFD1 (SE[EFD1]) (>= 0).  Typical
      value = 0,86.
    efd2: Exciter voltage at which exciter saturation is defined (EFD2) (> 0).  Typical value = 4,2.
    seefd2: Exciter saturation function value at the corresponding exciter voltage, EFD2 (SE[EFD2]) (>= 0).  Typical
      value = 0,5.
    """

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seefd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seefd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .Equipment import Equipment


@dataclass(config=DataclassConfig)
class ConductingEquipment(Equipment):
    """
    The parts of the AC power system that are designed to carry current or that are conductively connected through
      terminals.

    Terminals: Conducting equipment have terminals that may be connected to other conducting equipment terminals via
      connectivity nodes or topological nodes.
    BaseVoltage: Base voltage of this conducting equipment.  Use only when there is no voltage level container used and
      only one base voltage applies.  For example, not used for transformers.
    SvStatus: The status state variable associated with this conducting equipment.
    """

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # Terminals : list = Field(default_factory=list, in_profiles = [Profile.EQBD, Profile.EQ, Profile.DY, ]) # noqa: E501

    BaseVoltage: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # SvStatus : Optional[str] = Field(default=None, in_profiles = [Profile.SV, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .Equipment import Equipment


@dataclass(config=DataclassConfig)
class AuxiliaryEquipment(Equipment):
    """
    AuxiliaryEquipment describe equipment that is not performing any primary functions but support for the equipment
      performing the primary function. AuxiliaryEquipment is attached to primary equipment via an association with
      Terminal.

    Terminal: The Terminal at the equipment where the AuxiliaryEquipment is attached.
    """

    Terminal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .CrossCompoundTurbineGovernorDynamics import CrossCompoundTurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovSteamCC(CrossCompoundTurbineGovernorDynamics):
    """
    Cross compound turbine governor.  Unlike tandem compound units, cross compound units are not on the same shaft.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    pmaxhp: Maximum HP value position (Pmaxhp).  Typical value = 1.
    rhp: HP governor droop (Rhp) (> 0).  Typical value = 0,05.
    t1hp: HP governor time constant (T1hp) (>= 0).  Typical value = 0,1.
    t3hp: HP turbine time constant (T3hp) (>= 0).  Typical value = 0,1.
    t4hp: HP turbine time constant (T4hp) (>= 0).  Typical value = 0,1.
    t5hp: HP reheater time constant (T5hp) (>= 0).  Typical value = 10.
    fhp: Fraction of HP power ahead of reheater (Fhp).  Typical value = 0,3.
    dhp: HP damping factor (Dhp).  Typical value = 0.
    pmaxlp: Maximum LP value position (Pmaxlp).  Typical value = 1.
    rlp: LP governor droop (Rlp) (> 0).  Typical value = 0,05.
    t1lp: LP governor time constant (T1lp) (>= 0).  Typical value = 0,1.
    t3lp: LP turbine time constant (T3lp) (>= 0).  Typical value = 0,1.
    t4lp: LP turbine time constant (T4lp) (>= 0).  Typical value = 0,1.
    t5lp: LP reheater time constant (T5lp) (>= 0).  Typical value = 10.
    flp: Fraction of LP power ahead of reheater (Flp).  Typical value = 0,7.
    dlp: LP damping factor (Dlp).  Typical value = 0.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmaxhp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rhp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1hp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3hp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4hp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5hp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fhp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dhp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmaxlp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rlp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1lp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3lp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4lp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5lp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    flp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dlp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .EnergyConnection import EnergyConnection


@dataclass(config=DataclassConfig)
class EnergyConsumer(EnergyConnection):
    """
    Generic user of energy - a  point of consumption on the power system model. EnergyConsumer.pfixed, .qfixed,
      .pfixedPct and .qfixedPct have meaning only if there is no LoadResponseCharacteristic associated with
      EnergyConsumer or if LoadResponseCharacteristic.exponentModel is set to False.

    pfixed: Active power of the load that is a fixed quantity and does not vary as load group value varies. Load sign
      convention is used, i.e. positive sign means flow out from a node.
    pfixedPct: Fixed active power as a percentage of load group fixed active power. Used to represent the time-varying
      components.  Load sign convention is used, i.e. positive sign means flow out from a node.
    qfixed: Reactive power of the load that is a fixed quantity and does not vary as load group value varies. Load sign
      convention is used, i.e. positive sign means flow out from a node.
    qfixedPct: Fixed reactive power as a percentage of load group fixed reactive power. Used to represent the time-
      varying components.  Load sign convention is used, i.e. positive sign means flow out from a node.
    LoadResponse: The load response characteristic of this load.  If missing, this load is assumed to be constant power.
    p: Active power of the load. Load sign convention is used, i.e. positive sign means flow out from a node. For
      voltage dependent loads the value is at rated voltage. Starting value for a steady state solution.
    q: Reactive power of the load. Load sign convention is used, i.e. positive sign means flow out from a node. For
      voltage dependent loads the value is at rated voltage. Starting value for a steady state solution.
    LoadDynamics: Load dynamics model used to describe dynamic behaviour of this energy consumer.
    """

    pfixed: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    pfixedPct: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    qfixed: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    qfixedPct: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    LoadResponse: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    p: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    q: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    LoadDynamics: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEDC4B(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type DC4B model. These excitation systems utilize a field-controlled DC commutator exciter with a
      continuously acting voltage regulator having supplies obtained from the generator or auxiliary bus. Reference:
      IEEE 421.5-2005, 5.4.

    ka: Voltage regulator gain (KA) (> 0).  Typical value = 1.
    ta: Voltage regulator time constant (TA) (> 0).  Typical value = 0,2.
    kp: Regulator proportional gain (KP) (>= 0).  Typical value = 20.
    ki: Regulator integral gain (KI) (>= 0).  Typical value = 20.
    kd: Regulator derivative gain (KD) (>= 0).  Typical value = 20.
    td: Regulator derivative filter time constant (TD) (> 0 if ExcIEEEDC4B.kd > 0).  Typical value = 0,01.
    vrmax: Maximum voltage regulator output (VRMAX) (> ExcIEEEDC4B.vrmin).  Typical value = 2,7.
    vrmin: Minimum voltage regulator output (VRMIN) (<= 0 and < ExcIEEEDC4B.vrmax).  Typical value = -0,9.
    ke: Exciter constant related to self-excited field (KE).  Typical value = 1.
    te: Exciter time constant, integration rate associated with exciter control (TE) (> 0).  Typical value = 0,8.
    kf: Excitation control system stabilizer gain (KF) (>= 0).  Typical value = 0.
    tf: Excitation control system stabilizer time constant (TF) (>= 0).  Typical value = 1.
    efd1: Exciter voltage at which exciter saturation is defined (EFD1) (> 0).  Typical value = 1,75.
    seefd1: Exciter saturation function value at the corresponding exciter voltage, EFD1 (SE[EFD1]) (>= 0).  Typical
      value = 0,08.
    efd2: Exciter voltage at which exciter saturation is defined (EFD2) (> 0).  Typical value = 2,33.
    seefd2: Exciter saturation function value at the corresponding exciter voltage, EFD2 (SE[EFD2]) (>= 0).  Typical
      value = 0,27.
    vemin: Minimum exciter voltage output (VEMIN) (<= 0).  Typical value = 0.
    oelin: OEL input (OELin). true = LV gate false = subtract from error signal. Typical value = true.
    uelin: UEL input (UELin). true = HV gate false = add to error signal. Typical value = true.
    """

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    td: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seefd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seefd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vemin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    oelin: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    uelin: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcAC2A(ExcitationSystemDynamics):
    """
    Modified IEEE AC2A alternator-supplied rectifier excitation system with different field current limit.

    tb: Voltage regulator time constant (Tb) (>= 0).  Typical value = 0.
    tc: Voltage regulator time constant (Tc) (>= 0).  Typical value = 0.
    ka: Voltage regulator gain (Ka) (> 0).  Typical value = 400.
    ta: Voltage regulator time constant (Ta) (> 0).  Typical value = 0,02.
    vamax: Maximum voltage regulator output (Vamax) (> 0).  Typical value = 8.
    vamin: Minimum voltage regulator output (Vamin) (< 0).  Typical value = -8.
    kb: Second stage regulator gain (Kb) (> 0).  Exciter field current controller gain.  Typical value = 25.
    kb1: Second stage regulator gain (Kb1). It is exciter field current controller gain used as alternative to Kb to
      represent a variant of the ExcAC2A model.  Typical value = 25.
    vrmax: Maximum voltage regulator outputs (Vrmax) (> 0).  Typical value = 105.
    vrmin: Minimum voltage regulator outputs (Vrmin) (< 0).  Typical value = -95.
    te: Exciter time constant, integration rate associated with exciter control (Te) (> 0).  Typical value = 0,6.
    vfemax: Exciter field current limit reference (Vfemax) (>= 0).  Typical value = 4,4.
    kh: Exciter field current feedback gain (Kh) (>= 0).  Typical value = 1.
    kf: Excitation control system stabilizer gains (Kf) (>= 0).  Typical value = 0,03.
    kl: Exciter field current limiter gain (Kl).  Typical value = 10.
    vlr: Maximum exciter field current (Vlr) (> 0).  Typical value = 4,4.
    kl1: Coefficient to allow different usage of the model (Kl1).  Typical value = 1.
    ks: Coefficient to allow different usage of the model-speed coefficient (Ks) (>= 0).  Typical value = 0.
    tf: Excitation control system stabilizer time constant (Tf) (> 0).  Typical value = 1.
    kc: Rectifier loading factor proportional to commutating reactance (Kc) (>= 0).  Typical value = 0,28.
    kd: Demagnetizing factor, a function of exciter alternator reactances (Kd) (>= 0).  Typical value = 0,35.
    ke: Exciter constant related to self-excited field (Ke).  Typical value = 1.
    ve1: Exciter alternator output voltages back of commutating reactance at which saturation is defined (Ve1) (> 0).
      Typical value = 4,4.
    seve1: Exciter saturation function value at the corresponding exciter voltage, Ve1, back of commutating reactance
      (Se[Ve1]) (>= 0).  Typical value = 0,037.
    ve2: Exciter alternator output voltages back of commutating reactance at which saturation is defined (Ve2) (> 0).
      Typical value = 3,3.
    seve2: Exciter saturation function value at the corresponding exciter voltage, Ve2, back of commutating reactance
      (Se[Ve2]) (>= 0).  Typical value = 0,012.
    hvgate: Indicates if HV gate is active (HVgate). true = gate is used false = gate is not used. Typical value = true.
    lvgate: Indicates if LV gate is active (LVgate). true = gate is used false = gate is not used. Typical value = true.
    """

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kb: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kb1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vfemax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vlr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    hvgate: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    lvgate: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class DiagramObject(IdentifiedObject):
    """
    An object that defines one or more points in a given space. This object can be associated with anything that
      specializes IdentifiedObject. For single line diagrams such objects typically include such items as analog
      values, breakers, disconnectors, power transformers, and transmission lines.

    Diagram: A diagram object is part of a diagram.
    drawingOrder: The drawing order of this element. The higher the number, the later the element is drawn in sequence.
      This is used to ensure that elements that overlap are rendered in the correct order.
    isPolygon: Defines whether or not the diagram objects points define the boundaries of a polygon or the routing of a
      polyline. If this value is true then a receiving application should consider the first and last
      points to be connected.
    offsetX: The offset in the X direction. This is used for defining the offset from centre for rendering an icon (the
      default is that a single point specifies the centre of the icon).  The offset is in per-unit with 0
      indicating there is no offset from the horizontal centre of the icon.  -0.5 indicates it is offset by
      50% to the left and 0.5 indicates an offset of 50% to the right.
    offsetY: The offset in the Y direction. This is used for defining the offset from centre for rendering an icon (the
      default is that a single point specifies the centre of the icon).  The offset is in per-unit with 0
      indicating there is no offset from the vertical centre of the icon.  The offset direction is
      dependent on the orientation of the diagram, with -0.5 and 0.5 indicating an offset of +/- 50% on the
      vertical axis.
    rotation: Sets the angle of rotation of the diagram object.  Zero degrees is pointing to the top of the diagram.
      Rotation is clockwise.  DiagramObject.rotation=0 has the following meaning: The connection point of
      an element which has one terminal is pointing to the top side of the diagram. The connection point
      `From side` of an element which has more than one terminal is pointing to the top side of the
      diagram. DiagramObject.rotation=90 has the following meaning: The connection point of an element
      which has one terminal is pointing to the right hand side of the diagram. The connection point `From
      side` of an element which has more than one terminal is pointing to the right hand side of the
      diagram.
    IdentifiedObject: The domain object to which this diagram object is associated.
    DiagramObjectPoints: A diagram object can have 0 or more points to reflect its layout position, routing (for
      polylines) or boundary (for polygons).
    VisibilityLayers: A diagram object can be part of multiple visibility layers.
    DiagramObjectStyle: A diagram object has a style associated that provides a reference for the style used in the
      originating system.
    """

    Diagram: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DL,
        ],
    )

    drawingOrder: int = Field(
        default=0,
        in_profiles=[
            Profile.DL,
        ],
    )

    isPolygon: bool = Field(
        default=False,
        in_profiles=[
            Profile.DL,
        ],
    )

    offsetX: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DL,
        ],
    )

    offsetY: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DL,
        ],
    )

    rotation: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DL,
        ],
    )

    IdentifiedObject: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DL,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # DiagramObjectPoints : list = Field(default_factory=list, in_profiles = [Profile.DL, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # VisibilityLayers : list = Field(default_factory=list, in_profiles = [Profile.DL, ])

    DiagramObjectStyle: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DL,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .OverexcitationLimiterDynamics import OverexcitationLimiterDynamics


@dataclass(config=DataclassConfig)
class OverexcLim2(OverexcitationLimiterDynamics):
    """
    Different from LimIEEEOEL, LimOEL2 has a fixed pickup threshold and reduces the excitation set-point by means of a
      non-windup integral regulator. Irated is the rated machine excitation current (calculated from nameplate
      conditions: Vnom, Pnom, CosPhinom).

    koi: Gain Over excitation limiter (KOI).  Typical value = 0,1.
    voimax: Maximum error signal (VOIMAX) (> OverexcLim2.voimin).  Typical value = 0.
    voimin: Minimum error signal (VOIMIN) (< OverexcLim2.voimax).  Typical value = -9999.
    ifdlim: Limit value of rated field current (IFDLIM).  Typical value = 1,05.
    """

    koi: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    voimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    voimin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ifdlim: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcHU(ExcitationSystemDynamics):
    """
    Hungarian excitation system, with built-in voltage transducer.

    tr: Filter time constant (Tr) (>= 0). If a voltage compensator is used in conjunction with this excitation system
      model, Tr should be set to 0.  Typical value = 0,01.
    te: Major loop PI tag integration time constant (Te) (>= 0).  Typical value = 0,154.
    imin: Major loop PI tag output signal lower limit (Imin) (< ExcHU.imax).  Typical value = 0,1.
    imax: Major loop PI tag output signal upper limit (Imax) (> ExcHU.imin).  Typical value = 2,19.
    ae: Major loop PI tag gain factor (Ae).  Typical value = 3.
    emin: Field voltage control signal lower limit on AVR base (Emin) (< ExcHU.emax).  Typical value = -0,866.
    emax: Field voltage control signal upper limit on AVR base (Emax) (> ExcHU.emin).  Typical value = 0,996.
    ki: Current base conversion constant (Ki).  Typical value = 0,21428.
    ai: Minor loop PI tag gain factor (Ai).  Typical value = 22.
    ti: Minor loop PI control tag integration time constant (Ti) (>= 0).  Typical value = 0,01333.
    atr: AVR constant (Atr).  Typical value = 2,19.
    ke: Voltage base conversion constant (Ke).  Typical value = 4,666.
    """

    tr: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    imin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    imax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ae: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    emin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    emax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ai: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ti: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    atr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcSEXS(ExcitationSystemDynamics):
    """
    Simplified excitation system.

    tatb: Gain reduction ratio of lag-lead element ([Ta / Tb]).  Typical value = 0,1.
    tb: Denominator time constant of lag-lead block (Tb) (>= 0).  Typical value = 10.
    k: Gain (K) (> 0).  Typical value = 100.
    te: Time constant of gain block (Te) (> 0).  Typical value = 0,05.
    emin: Minimum field voltage output (Emin) (< ExcSEXS.emax).  Typical value = -5.
    emax: Maximum field voltage output (Emax) (> ExcSEXS.emin).  Typical value = 5.
    kc: PI controller gain (Kc) (> 0 if ExcSEXS.tc > 0).  Typical value = 0,08.
    tc: PI controller phase lead time constant (Tc) (>= 0).  Typical value = 0.
    efdmin: Field voltage clipping minimum limit (Efdmin) (< ExcSEXS.efdmax).  Typical value = -5.
    efdmax: Field voltage clipping maximum limit (Efdmax) (> ExcSEXS.efdmin).  Typical value = 5.
    """

    tatb: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    emin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    emax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class NonlinearShuntCompensatorPoint(Base):
    """
    A non linear shunt compensator bank or section admittance value. The number of NonlinearShuntCompenstorPoint
      instances associated with a NonlinearShuntCompensator shall be equal to ShuntCompensator.maximumSections.
      ShuntCompensator.sections shall only be set to one of the NonlinearShuntCompenstorPoint.sectionNumber. There
      is no interpolation between NonlinearShuntCompenstorPoint-s.

    NonlinearShuntCompensator: Non-linear shunt compensator owning this point.
    b: Positive sequence shunt (charging) susceptance per section.
    g: Positive sequence shunt (charging) conductance per section.
    sectionNumber: The number of the section.
    b0: Zero sequence shunt (charging) susceptance per section.
    g0: Zero sequence shunt (charging) conductance per section.
    """

    NonlinearShuntCompensator: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    b: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    g: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    sectionNumber: int = Field(
        default=0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    b0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    g0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SC,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcAC6A(ExcitationSystemDynamics):
    """
    Modified IEEE AC6A alternator-supplied rectifier excitation system with speed input.

    ka: Voltage regulator gain (Ka) (> 0).  Typical value = 536.
    ks: Coefficient to allow different usage of the model-speed coefficient (Ks).  Typical value = 0.
    ta: Voltage regulator time constant (Ta) (>= 0).  Typical value = 0,086.
    tk: Voltage regulator time constant (Tk) (>= 0).  Typical value = 0,18.
    tb: Voltage regulator time constant (Tb) (>= 0).  Typical value = 9.
    tc: Voltage regulator time constant (Tc) (>= 0).  Typical value = 3.
    vamax: Maximum voltage regulator output (Vamax) (> 0).  Typical value = 75.
    vamin: Minimum voltage regulator output (Vamin) (< 0).  Typical value = -75.
    vrmax: Maximum voltage regulator output (Vrmax) (> 0).  Typical value = 44.
    vrmin: Minimum voltage regulator output (Vrmin) (< 0).  Typical value = -36.
    te: Exciter time constant, integration rate associated with exciter control (Te) (> 0).  Typical value = 1.
    kh: Exciter field current limiter gain (Kh) (>= 0).  Typical value = 92.
    tj: Exciter field current limiter time constant (Tj) (>= 0).  Typical value = 0,02.
    th: Exciter field current limiter time constant (Th) (> 0).  Typical value = 0,08.
    vfelim: Exciter field current limit reference (Vfelim) (> 0).  Typical value = 19.
    vhmax: Maximum field current limiter signal reference (Vhmax) (> 0).  Typical value = 75.
    kc: Rectifier loading factor proportional to commutating reactance (Kc) (>= 0).  Typical value = 0,173.
    kd: Demagnetizing factor, a function of exciter alternator reactances (Kd) (>= 0).  Typical value = 1,91.
    ke: Exciter constant related to self-excited field (Ke).  Typical value = 1,6.
    ve1: Exciter alternator output voltages back of commutating reactance at which saturation is defined (Ve1) (> 0).
      Typical value = 7,4.
    seve1: Exciter saturation function value at the corresponding exciter voltage, Ve1, back of commutating reactance
      (Se[Ve1]) (>= 0).  Typical value = 0,214.
    ve2: Exciter alternator output voltages back of commutating reactance at which saturation is defined (Ve2) (> 0).
      Typical value = 5,55.
    seve2: Exciter saturation function value at the corresponding exciter voltage, Ve2, back of commutating reactance
      (Se[Ve2]) (>= 0).  Typical value = 0,044.
    """

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tk: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tj: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    th: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vfelim: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vhmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }
